@@ -18,9 +18,9 @@
 ;;; Notes:
 
 ;; @table code
-;; @item mspec->fh-wrapper
+;; @item mdecl->fh-wrapper
 ;; generates code to apply wrapper to objects returned from foreign call
-;; @item mspec->fh-unwrapper
+;; @item mdecl->fh-unwrapper
 ;; generated code to apply un-wrapper to arguments for foreign call
 ;; @end table
 
@@ -30,6 +30,18 @@
 
 ;; TODO: add renamer
 
+;; Issue:
+;; So issue is when 'typedef struct ref foo_t' has no 'struct def'
+;; we never define a type.  Then later we may see 'typedef foo_t bar_t'
+;; We are using define-ffi-type-alias but that then generates a reference
+;; to an undefined type.  Maybe for the above we should have a void
+;; pseudo-type with
+;; name: void
+;; (unwrap-void obj) => 'void
+;; (wrap 'void) (make-xxx)
+;; (pointer-to obj) => <void* obj>
+;; (value-at void*-object) =. void
+
 ;; For enum typedefs we are not creating types but just using wrappers.
 
 ;;; Code:
@@ -50,7 +62,6 @@
 	    )
   #:use-module (nyacc lang c99 cpp)
   #:use-module (nyacc lang c99 parser)
-  ;;#:use-module (nyacc lang c99 xparser)
   #:use-module (nyacc lang c99 pprint)
   #:use-module (nyacc lang c99 munge)
   #:use-module (nyacc lang c99 cxeval)
@@ -129,8 +140,8 @@
 (define *renamer* (make-parameter identity)) ; renamer from ffi-module
 (define *errmsgs* (make-parameter '()))	     ; list of warnings
 ;; what about option to trace
+(define *echo-decls* (make-parameter #f)) ; add echo-decls code for debugging
 
-(define *echo-decls* #f)		; add echo-decls code for debugging
 
 (define (sfscm fmt . args)
   (apply simple-format (*mport*) fmt args))
@@ -322,7 +333,7 @@
      `(define ,(link-libs)
 	(list ,@(map (lambda (l) `(dynamic-link ,l)) (reverse libraries)))))
     ;;(sfscm "(define link-lib (car link-libs))\n")
-    (if *echo-decls* (sfscm "(define echo-decls #t)\n"))))
+    (if (*echo-decls*) (sfscm "(define echo-decls #t)\n"))))
 
 ;; === type conversion ==============
 
@@ -400,9 +411,9 @@
 
 ;; just the type, so parent has to build the name-value pairs for
 ;; struct members
-(define (mtail->bs-desc mspec-tail)
+(define (mtail->bs-desc mdecl-tail)
   (let ((defined (*defined*))) ;; (udict (*udict*)))
-    (pmatch mspec-tail
+    (pmatch mdecl-tail
       ;; expand typeref, use renamer,
       (((typename ,name))
        (or (assoc-ref bs-typemap name)
@@ -490,8 +501,8 @@
        `(bit-field ,(const-expr->number size) ,(mtail->bs-desc rest)))
 
       (,otherwise
-       (sferr "mtail->bs-desc missed mspec:\n")
-       (pperr mspec-tail)
+       (sferr "mtail->bs-desc missed mdecl:\n")
+       (pperr mdecl-tail)
        ;;(error "quit") (quit)
        (fherr "mtail->bs-desc failed")
        )
@@ -614,7 +625,7 @@
 		 (udecl (cdar decls))
 		 ;; fix the following, look at cleanup-udecl
 		 (udecl (udecl-rem-type-qual udecl)) ;; remove "const" "extern"
-		 (spec (udecl->mspec/comm udecl))
+		 (spec (udecl->mdecl/comm udecl))
 		 (tail (cddr spec))
 		 (type (mtail->bs-desc tail)))
 	    (cond
@@ -766,10 +777,10 @@
     (ffi:uint64 . ,uint64) (ffi-void* . *)
     ))
 
-(define (mtail->ffi-desc mspec-tail)
-  ;;(sferr "mspec=~S\n" mspec)
-  (if (and (pair? mspec-tail) (string? (car mspec-tail))) (error "xxx"))
-  (pmatch mspec-tail
+(define (mtail->ffi-desc mdecl-tail)
+  ;;(sferr "mdecl=~S\n" mdecl)
+  (if (and (pair? mdecl-tail) (string? (car mdecl-tail))) (error "xxx"))
+  (pmatch mdecl-tail
     (((pointer-to) . ,rest) 'ffi-void*)
     (((array-of) . ,rest) 'ffi-void*)
     (((array-of ,size) . ,rest) 'ffi-void*)
@@ -792,8 +803,8 @@
 			    (name (caar udict))
 			    (udecl (cdar udict))
 			    (udecl (udecl-rem-type-qual udecl))
-			    (mspec (udecl->mspec udecl)))
-		       (mtail->ffi-desc (cdr mspec))))
+			    (mdecl (udecl->mdecl udecl)))
+		       (mtail->ffi-desc (cdr mdecl))))
 		   fields)))
     (((struct-def (ident ,name) ,field-list))
      (mtail->ffi-desc `((struct-def ,field-list))))
@@ -807,8 +818,8 @@
 	   (let* ((udict (unitize-comp-decl (car flds)))
 		  (udecl (cdar udict))
 		  (udecl (udecl-rem-type-qual udecl))
-		  (mspec (udecl->mspec udecl))
-		  (ftype (mtail->ffi-desc (cdr mspec)))
+		  (mdecl (udecl->mdecl udecl))
+		  (ftype (mtail->ffi-desc (cdr mdecl)))
 		  (ftval (assq-ref ffi-symmap ftype))
 		  (fsize (sizeof ftval)))
 	     (if (> fsize size)
@@ -818,21 +829,21 @@
      (mtail->ffi-desc `((union-def ,field-list))))
     
     (,otherwise
-     (sferr "mtail->ffi-desc missed:\n") (pperr mspec-tail) ;;(quit)
-     (error "") (fherr "mtail->ffi-desc missed: ~S" mspec-tail))))
+     (sferr "mtail->ffi-desc missed:\n") (pperr mdecl-tail) ;;(quit)
+     (error "") (fherr "mtail->ffi-desc missed: ~S" mdecl-tail))))
 
-;; Return a mspec for the return type.  The variable is called @code{NAME}.
+;; Return a mdecl for the return type.  The variable is called @code{NAME}.
 (define (gen-decl-return udecl)
   (let* ((udecl1 (expand-typerefs udecl (*udict*) ffi-defined))
 	 (udecl (udecl-rem-type-qual udecl1))
-	 (mspec (udecl->mspec udecl1)))
-    (mtail->ffi-desc (cdr mspec))))
+	 (mdecl (udecl->mdecl udecl1)))
+    (mtail->ffi-desc (cdr mdecl))))
 
 (define (gen-bs-decl-return udecl)
   (let* ((udecl1 (expand-typerefs udecl (*udict*) ffi-defined))
 	 (udecl (udecl-rem-type-qual udecl1))
-	 (mspec (udecl->mspec udecl1)))
-    (mtail->bs-desc (cdr mspec))))
+	 (mdecl (udecl->mdecl udecl1)))
+    (mtail->bs-desc (cdr mdecl))))
 
 (define (int->abs-ident ix)
   (simple-format #f "arg-~A" ix))
@@ -849,9 +860,9 @@
       ;;(sferr "\nP: ~S\n" (car params))
       (let* ((udecl1 (expand-typerefs (car params) (*udict*) ffi-defined))
 	     (udecl1 (udecl-rem-type-qual udecl1))
-	     (mspec (udecl->mspec udecl1 #:abs-ident (int->abs-ident ix))))
+	     (mdecl (udecl->mdecl udecl1 #:abs-ident (int->abs-ident ix))))
 	;;(sferr "  ~S\n" udecl1)
-	(cons (mtail->ffi-desc (cdr mspec))
+	(cons (mtail->ffi-desc (cdr mdecl))
 	      (iter (1+ ix) (cdr params))))))))
 
 (define (gen-bs-decl-params params)
@@ -864,18 +875,18 @@
      (else
       (let* ((udecl1 (expand-typerefs (car params) (*udict*) ffi-defined))
 	     (udecl1 (udecl-rem-type-qual udecl1))
-	     (mspec (udecl->mspec udecl1 #:abs-ident (int->abs-ident ix))))
-	(cons (mtail->bs-desc (cdr mspec))
+	     (mdecl (udecl->mdecl udecl1 #:abs-ident (int->abs-ident ix))))
+	(cons (mtail->bs-desc (cdr mdecl))
 	      (iter (1+ ix) (cdr params))))))))
 
 ;; === function calls : unwrap args, call, wrap return
 
-;; given mspec for an exec argument give the unwrapper
-(define (mspec->fh-unwrapper mspec)
-  ;;(sferr "mspec:\n") (pperr mspec)
+;; given mdecl for an exec argument give the unwrapper
+(define (mdecl->fh-unwrapper mdecl)
+  ;;(sferr "mdecl:\n") (pperr mdecl)
   (let ((wrapped (*wrapped*)) (defined (*defined*)))
     ;; git_reference_foreach_name_cb not preserved
-    (pmatch (cdr mspec)
+    (pmatch (cdr mdecl)
       (((fixed-type ,name)) 'unwrap~fixed)
       (((float-type ,name)) 'unwrap~float)
       (((void)) #f)
@@ -921,12 +932,12 @@
 	(else 'unwrap~pointer)))
 
       (((pointer-to) (function-returning (param-list . ,params)) . ,rest)
-       (let* ((udecl (mspec->udecl (cons "~ret" rest)))
+       (let* ((udecl (mdecl->udecl (cons "~ret" rest)))
 	      (udecl (expand-typerefs udecl (*udict*) ffi-defined))
-	      (mspec (udecl->mspec udecl))
-	      (decl-return (mtail->ffi-desc (cdr mspec)))
+	      (mdecl (udecl->mdecl udecl))
+	      (decl-return (mtail->ffi-desc (cdr mdecl)))
 	      (decl-params (gen-decl-params params)))
-	 ;;(sferr "FIX RET => ~S\n" mspec)
+	 ;;(sferr "FIX RET => ~S\n" mdecl)
 	 (if (and (pair? decl-params) (equal? (last decl-params) '...))
 	     (fherr/once "no varargs (yet)"))
 	 `(make-fctn-param-unwrapper ,decl-return (list ,@decl-params))))
@@ -939,12 +950,12 @@
       (((array-of) . ,rest) 'unwrap~array)
 
       (,otherwise
-       (sferr "mspec->fh-unwrapper missed:\n") (pperr mspec) (quit)
-       (fherr "mspec->fh-unwrapper missed: ~S" mspec)))))
+       (sferr "mdecl->fh-unwrapper missed:\n") (pperr mdecl) (quit)
+       (fherr "mdecl->fh-unwrapper missed: ~S" mdecl)))))
 
-(define (mspec->fh-wrapper mspec)
+(define (mdecl->fh-wrapper mdecl)
   (let ((wrapped (*wrapped*)) (defined (*defined*)))
-    (pmatch (cdr mspec)
+    (pmatch (cdr mdecl)
       (((fixed-type ,name)) (if (assoc-ref ffi-typemap name) #f
 				(fherr "todo: ffi-wrap fixed")))
       (((float-type ,name)) (if (assoc-ref ffi-typemap name) #f
@@ -992,7 +1003,7 @@
       ;;(((pointer-to) . ,otherwise) 'ffi:make-pointer)
       (((pointer-to) . ,otherwise) #f)
 
-      (,otherwise (fherr "mspec->fh-wrapper missed: ~S" mspec)))))
+      (,otherwise (fherr "mdecl->fh-wrapper missed: ~S" mdecl)))))
 
 ;; given list of udecl params generate list of name-unwrap pairs
 (define (gen-exec-params params)
@@ -1006,8 +1017,8 @@
        ;; then we will need to create enum types in cnvt-udecl typedefs.
        (let* ((param-decl (expand-typerefs param-decl (*udict*) (*wrapped*)))
 	      (param-decl (udecl-rem-type-qual param-decl)) ;; ???
-	      (mspec (udecl->mspec param-decl)))
-	 (acons (car mspec) (mspec->fh-unwrapper mspec) seed)))))
+	      (mdecl (udecl->mdecl param-decl)))
+	 (acons (car mdecl) (mdecl->fh-unwrapper mdecl) seed)))))
    '() params))
 
 ;; given list of name-unwrap pairs generate function arg names
@@ -1040,8 +1051,8 @@
 (define (gen-exec-return-wrapper udecl)
   (let* ((udecl (expand-typerefs udecl (*udict*) (*wrapped*)))
 	 (udecl (udecl-rem-type-qual udecl))
-	 (mspec (udecl->mspec udecl)))
-    (mspec->fh-wrapper mspec)))
+	 (mdecl (udecl->mdecl udecl)))
+    (mdecl->fh-wrapper mdecl)))
 
 (define (fix-params param-decls)
 
@@ -1120,8 +1131,8 @@
     (sfscm ";; (~A) => bytestructure\n" name)
     (ppscm
      `(define-public ,(string->symbol name)
-	(let ((bstr-promise (delay (fh-link-bstr ,name ,desc ,(link-libs)))))
-	  (lambda () (force bstr-promise)))))))
+	(let ((x-promise (delay (fh-link-extern ,name ,desc ,(link-libs)))))
+	  (lambda () (force x-promise)))))))
 
 ;; ------------------------------------
 
@@ -1187,7 +1198,7 @@
 ;; Given udecl produce a ffi-spec.
 ;; Return updated (string based) keep-list, which will be modified if the
 ;; declaration is a typedef.  The typelist is the set of keepers used for
-;; @code{udecl->mspec}.
+;; @code{udecl->mdecl}.
 ;; Returns values wrapped, defined.
 ;; @end deffn
 ;; NOT SURE WHAT defined MEANS NOW
@@ -1403,6 +1414,12 @@
 	(else
 	 ;; 3) struct never defined; only used as pointer
 	 (sfscm "(define-public ~A-desc 'void)\n" typename)
+	 ;; ----- This may be a hack? Write it up! ------------
+	 ;; Added to get glugl GLUnurbsObj to work.
+	 (sfscm "(define-fh-type-alias ~A fh-void)\n" typename)
+	 (sfscm "(define-public ~A? fh-void?)\n" typename)
+	 (sfscm "(define-public make-~A make-fh-void)\n" typename)
+	 ;; --------------------------------------------------
 	 (sfscm "(define-public ~A*-desc (bs:pointer ~A-desc))\n"
 		typename typename)))
        (fhscm-def-pointer (sw/* typename))
@@ -1553,11 +1570,15 @@
       ;; TODO: typedef void* (foo_t)(int x)
 
       ;; typedef foo_t bar_t
+      ;; We retry with expansion of foo_t here.  Using fh-define-type-alias
+      ;; was not working when we had "typedef struct foo foo_t;" But then
+      ;; crashing on function types, so imported original type aliasing.
       ((udecl
 	(decl-spec-list
 	 (stor-spec (typedef))
 	 (type-spec (typename ,name)))
 	(init-declr (ident ,typename)))
+       ;;(sferr "typedef ~S =>\n" typename) (pperr (udict-ref udict name))
        (cond
 	((member name bs-defined)
 	 (values wrapped defined))
@@ -1698,8 +1719,8 @@
        ;; This needs to have a delay and handler
        (let* ((udecl (expand-typerefs udecl udict (*defined*)))
 	      (udecl (udecl-rem-type-qual udecl))
-	      (mspec (udecl->mspec udecl)))
-	 (cnvt-extern (car mspec) (cdr mspec)))
+	      (mdecl (udecl->mdecl udecl)))
+	 (cnvt-extern (car mdecl) (cdr mdecl)))
        (values wrapped defined))
 
       ;; non-pointer
@@ -1707,8 +1728,8 @@
 	      ,init-declr . ,rest)
        (let* ((udecl (expand-typerefs udecl udict (*defined*)))
 	      (udecl (udecl-rem-type-qual udecl))
-	      (mspec (udecl->mspec udecl)))
-	 (cnvt-extern (car mspec) (cdr mspec))
+	      (mdecl (udecl->mdecl udecl)))
+	 (cnvt-extern (car mdecl) (cdr mdecl))
 	 (values wrapped defined)))
 
       ;; === special cases I need to fix =
@@ -1812,7 +1833,6 @@
 			    (with-input-from-string ""
 			      (lambda ()
 				(expand-cpp-macro-ref name cpp-defs)))))))
-	       ;;(if (string=? name "DBUS_TYPE_BOOLEAN") (sferr "~S\n" repl))
 	       ;; TODO: try to reduce all this to the parse-c99x one
 	       (cond
 		((not repl) seed)
@@ -1820,17 +1840,6 @@
 		((zero? (string-length repl)) seed)
 		;;
 		((cintstr->num repl) => (lambda (val) (acons symb val seed)))
-		#|
-		((string=? "\"\0\"" repl) (acons symb repl seed)) ;; hack
-		((eqv? #\" (string-ref repl 0))
-		 ;; This breaks when a string contains #\nul
-		 (acons symb
-			(regexp-substitute/global ;; "abc" "def" => "abcdef"
-			 #f "\"\\s*\""
-			 (substring repl 1 (1- (string-length repl)))
-			 'pre 'post)
-			seed))
-		|#
 		((with-error-to-port (open-output-file "/dev/null")
 		   (lambda ()
 		     (catch 'c99-error
@@ -1938,25 +1947,25 @@
 		 (stat:mtimensec stat2))))))
 
 ;; given modules spec, determine if any ffi-module dependencies are
-;; outdated 
+;; outdated
 (define (check-deps module-options)
   (let ((ext-modz ;; use filter?
 	 (fold-right
 	  (lambda (opt seed)
 	    (if (eq? (car opt) 'use-ffi-module) (cons (cdr opt) seed) seed))
 	  '() module-options))
-	(ext-mods (map cdr (filter (lambda (opt) (eq? (car opt) 'use-ffi-module))
-				   module-options)))
-	)
+	(ext-mods (map cdr (filter
+			    (lambda (opt) (eq? (car opt) 'use-ffi-module))
+			    module-options))))
     (for-each
      (lambda (fmod)
        (let* ((base (string-join (map symbol->string fmod) "/"))
 	      (xffi (string-append base ".ffi"))
 	      (xscm (string-append base ".scm")))
 	 (when (not (access? xscm R_OK))
-	   (fherr "compiled dependent ~S not found\n" fmod))
+	   (fherr "compiled dependent ~S not found" fmod))
 	 (when (more-recent? xffi xscm)
-	   (fherr "dependent ~S needs recompile\n" xffi)
+	   (fherr "dependent ~S needs recompile" xffi)
 	   (sleep 2))))
      ext-mods)))
 
@@ -1978,7 +1987,7 @@
 			(string=? "*anon*" (cdr name)))))
 	     (let ((udecl (udict-ref udict name)))
 	       (nlscm) (c99scm udecl)
-	       (if *echo-decls*
+	       (if (*echo-decls*)
 		   (sfscm "(if echo-decls (display \"~A\\n\"))\n" name))
 	       (cnvt-udecl udecl udict wrapped defined)))
 	    (else (values wrapped defined))))
@@ -2050,7 +2059,7 @@
 	   (lambda (upath seed)
 	     (unless (resolve-module upath)
 	       (error "module not defined:" upath))
-	     (let* ((modul (resolve-module upath))
+	     (let* ((modul (resolve-module upath #:ensure #f))
 		    (pname (path->name upath))
 		    (vname (string->symbol (string-append pname "-types")))
 		    (var (module-ref modul vname)))
@@ -2063,6 +2072,7 @@
     (*mport* mport)
     (*ddict* ddict)
     (*tdefs* (udict->typenames udict))
+    (if (assq-ref script-options 'debug) (*echo-decls* #t))
     ;; renamer?
 
     ;; file and module header
@@ -2290,29 +2300,29 @@
 ;; This procedure will 
 ;; @end deffn
 (define* (compile-ffi-file file #:optional (options '()))
-  (parameterize ((*options* '()) (*wrapped* '()) (*defined* '())
+  (parameterize ((*options* options) (*wrapped* '()) (*defined* '())
 		 (*renamer* identity) (*errmsgs* '()) (*prefix* "")
 		 (*mport* #t) (*udict* '()) (*ddict* '()))
-    (sferr "ffi-help: WARNING: the FFI helper is experimental\n")
     ;; if not interactive ...
     (debug-disable 'backtrace)
     (if (not (access? file R_OK))
 	(throw 'ffi-help-error "ERROR: not found: ~S" file))
     (call-with-input-file file
       (lambda (iport)
-	(let iter ((oport #f) (exp (read iport)))
-	  (cond
-	   ((eof-object? exp)
-	    (when oport
-	      (display "\n;; --- last line ---\n" oport)
-	      (sferr "wrote `~A'\n" (port-filename oport))
-	      (close-port oport)))
-	   ((and (pair? exp) (eqv? 'define-ffi-module (car exp)))
-	    (iter (eval exp (current-module)) (read iport)))
-	   (else
-	    (when oport
-	      (newline oport)
-	      (pretty-print exp oport))
-	    (iter oport (read iport)))))))))
+	(let ((env (make-fresh-user-module)))
+	  (eval '(use-modules (nyacc lang c99 ffi-help)) env)
+	  (let iter ((oport #f) (exp (read iport)))
+	    (cond
+	     ((eof-object? exp)
+	      (when oport
+		(display "\n;; --- last line ---\n" oport)
+		(close-port oport)))
+	     ((and (pair? exp) (eqv? 'define-ffi-module (car exp)))
+	      (iter (eval exp env) (read iport)))
+	     (else
+	      (when oport
+		(newline oport)
+		(pretty-print exp oport))
+	      (iter oport (read iport))))))))))
 
 ;; --- last line ---
