@@ -19,12 +19,21 @@
 
 (define-module (system ffi-help-rt)
   #:export (*ffi-help-version*
+
+	    ;; user level routines
 	    fh-type?
 	    fh-object? fh-object-val
 	    fh-object-ref fh-object-set!
 	    pointer-to value-at NULL
 
-	    fh:function fh:cast fh-cast bs-addr
+	    ;; maybe used outside of modules?
+	    fh:cast fh-cast bs-addr
+	    fh:function
+	    ffi-void*
+	    fh-void fh-void? make-fh-void
+	    make-fht
+
+	    ;; called from output of the ffi-compiler
 	    define-fh-pointer-type
 	    define-fh-type-alias
 	    define-fh-compound-type
@@ -32,20 +41,23 @@
 	    define-fh-function*-type
 	    ref<->deref! fh-ref<->deref!
 	    make-symtab-function
-
 	    fh-find-symbol-addr
 	    fht-wrap fht-unwrap fh-wrap fh-unwrap
 	    unwrap~fixed unwrap~float
 	    unwrap~pointer unwrap~array
 	    make-fctn-param-unwrapper
-	    fh-link-proc fh-link-bstr ffi-void*)
+	    fh-link-proc fh-link-extern
+	    
+	    ;; deprecated
+	    fh-link-bstr ;; => fh-link-extern
+	    )
   #:use-module (bytestructures guile)
   #:use-module (bytestructures guile ffi)
   #:use-module (rnrs bytevectors)
   #:use-module ((system foreign) #:prefix ffi:)
   #:use-module (srfi srfi-9))
 
-(define *ffi-help-version* "0.86.2")
+(define *ffi-help-version* "0.86.3")
 
 (define (sferr fmt . args) (apply simple-format (current-error-port) fmt args))
 
@@ -113,9 +125,9 @@
 
 ;; execute the type method on the object
 (define (fh-unwrap type obj)
-  ((fht-unwrap type)) obj)
+  ((fht-unwrap type) obj))
 (define (fh-wrap type val)
-  ((fht-wrap type)) val)
+  ((fht-wrap type) val))
 
 ;; Right now this returns a ffi pointer.
 ;; TODO: add field option so we can do (pointer-to xstr 'vec)
@@ -599,6 +611,19 @@
   (ffi:pointer->string (ffi:make-pointer (fh-object-ref obj))))
 (export make-char* char*->string)
 
+(define fh-void
+  (make-fht 'void 
+	    (lambda (obj) 'void)
+	    (lambda (val) (make-struct/no-tail fh-void val))
+	    #f #f
+	    (lambda (obj port) (display "#<fh-void>" port))))
+(define fh-void?
+  (lambda (obj) (eq? (struct-vtable obj) fh-void)))
+(define make-fh-void
+  (case-lambda
+    (() (make-struct/no-tail fh-void 'void))
+    ((val) (make-struct/no-tail fh-void 'void))))
+
 (define-syntax make-maker
   (syntax-rules ()
     ((_ desc make-bs-obj)
@@ -687,12 +712,16 @@
   (let ((dfunc (fh-find-symbol-addr name dl-lib-list)))
     (and dfunc (ffi:pointer->procedure return dfunc args))))
 
-;; @deffn {Procedure} fh-link-bstr name desc db-lib-list => bytestructure
+;; @deffn {Procedure} fh-link-extern name desc db-lib-list => bs
 ;; Generate a bytestructure from the bytes in the library at the var addr.
 ;; @end deffn
-(define* (fh-link-bstr name desc dl-lib-list)
+(define* (fh-link-extern name desc dl-lib-list)
   (let* ((addr (fh-find-symbol-addr name dl-lib-list))
 	 (size (bytestructure-descriptor-size desc)))
     (make-bytestructure (ffi:pointer->bytevector addr size) 0 desc)))
 
+;; === deprecated ==============================================================
+
+(define fh-link-bstr fh-link-extern)
+
 ;; --- last line ---
